const std = @import("std");
const zbs = std.build;

pub fn build(b: *zbs.Builder) void {
    // Standard target options allows the person running `zig build` to choose
    // what target to build for. Here we do not override the defaults, which
    // means any target is allowed, and the default is native. Other options
    // for restricting supported target set are available.
    const target = b.standardTargetOptions(.{});

    // Standard release options allow the person running `zig build` to select
    // between Debug, ReleaseSafe, ReleaseFast, and ReleaseSmall.
    const mode = b.standardReleaseOptions();

    const cmph = b.addStaticLibrary("cmph", null);
    cmph.setTarget(target);
    cmph.setBuildMode(mode);
    cmph.linkLibC();
    cmph.force_pic = true;
    cmph.addCSourceFiles(&.{
        "deps/cmph/src/bdz.c",
        "deps/cmph/src/bdz_ph.c",
        "deps/cmph/src/bmz8.c",
        "deps/cmph/src/bmz.c",
        "deps/cmph/src/brz.c",
        "deps/cmph/src/buffer_entry.c",
        "deps/cmph/src/buffer_manager.c",
        "deps/cmph/src/chd.c",
        "deps/cmph/src/chd_ph.c",
        "deps/cmph/src/chm.c",
        "deps/cmph/src/cmph.c",
        "deps/cmph/src/cmph_structs.c",
        "deps/cmph/src/compressed_rank.c",
        "deps/cmph/src/compressed_seq.c",
        "deps/cmph/src/fch_buckets.c",
        "deps/cmph/src/fch.c",
        "deps/cmph/src/graph.c",
        "deps/cmph/src/hash.c",
        "deps/cmph/src/jenkins_hash.c",
        "deps/cmph/src/linear_string_map.c",
        "deps/cmph/src/miller_rabin.c",
        "deps/cmph/src/select.c",
        "deps/cmph/src/vqueue.c",
    }, &.{
        "-W",
        "-Wno-unused-function",
        "-fomit-frame-pointer",
        //"-DDEBUG",
    });
    cmph.addIncludeDir("deps/cmph/src");
    cmph.addIncludeDir("include/deps/cmph");

    {
        const exe = b.addExecutable("turbo-unix2db", "cli/unix2db/main.zig");
        exe.setTarget(target);
        exe.setBuildMode(mode);
        addCmphDeps(exe, cmph);
        exe.install();
    }

    {
        const src_test = b.addTest("src/test_all.zig");
        addCmphDeps(src_test, cmph);
        const test_step = b.step("test", "Run the tests");
        test_step.dependOn(&src_test.step);
    }
}

fn addCmphDeps(exe: *zbs.LibExeObjStep, cmph: *zbs.LibExeObjStep) void {
    exe.linkLibC();
    exe.linkLibrary(cmph);
    exe.addIncludeDir("deps/cmph/src");
}
